import CaptionAverageIcon from 'assets/icons/visu/duel/captionAverage.svg'
import CaptionConsumptionIcon from 'assets/icons/visu/duel/captionConsumption.svg'
import CaptionIncomingIcon from 'assets/icons/visu/duel/captionIncoming.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import DuelChart from 'components/Duel/DuelChart'
import DuelResultModal from 'components/Duel/DuelResultModal'
import LastDuelModal from 'components/Duel/lastDuelModal'
import { Client, useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { UsageEventType } from 'enum/usageEvent.enum'
import {
  UserChallengeSuccess,
  UserChallengeUpdateFlag,
} from 'enum/userChallenge.enum'
import { UserDuelState } from 'enum/userDuel.enum'
import { Dataload, UserChallenge, UserDuel } from 'models'
import React, {
  Dispatch,
  useCallback,
  useEffect,
  useMemo,
  useRef,
  useState,
} from 'react'
import { useDispatch, useSelector } from 'react-redux'
import { useNavigate } from 'react-router-dom'
import ChallengeService from 'services/challenge.service'
import UsageEventService from 'services/usageEvent.service'
import { AppActionsTypes, AppStore } from 'store'
import {
  unlockNextUserChallenge,
  updateUserChallengeList,
} from 'store/challenge/challenge.actions'
import { toggleChallengeDuelNotification } from 'store/global/global.actions'
import { formatNumberValues } from 'utils/utils'
import './duelOngoing.scss'

interface DuelOngoingProps {
  userChallenge: UserChallenge
  isFinished?: boolean
}

const DuelOngoing: React.FC<DuelOngoingProps> = ({
  userChallenge,
  isFinished,
}: DuelOngoingProps) => {
  const client: Client = useClient()
  const { t } = useI18n()
  const { currentDataload, userChallengeList } = useSelector(
    (state: AppStore) => state.ecolyo.challenge
  )
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const navigate = useNavigate()
  const [resultModal, setResultModal] = useState<boolean>(false)
  const [winChallenge, setWinChallenge] = useState<boolean>(false)
  const [isLastDuel, setIsLastDuel] = useState<boolean>(false)
  const [width, setWidth] = useState<number>(0)
  const [height, setHeight] = useState<number>(0)
  const [finishedDataLoad, setFinishedDataLoad] = useState<Dataload[]>()
  const chartContainer = useRef<HTMLDivElement>(null)
  const challengeService = useMemo(() => new ChallengeService(client), [client])

  const duel: UserDuel = userChallenge.duel
  const title: string = duel.title
  const durationInDays: number = duel.duration.days

  const userConsumption: string = formatNumberValues(
    userChallenge.duel.userConsumption
  ).toString()
  const average: string = formatNumberValues(
    userChallenge.duel.threshold
  ).toString()

  const setResult = useCallback(async () => {
    const challengeService = new ChallengeService(client)
    const updatedChallenge = await challengeService.updateUserChallenge(
      userChallenge,
      winChallenge
        ? UserChallengeUpdateFlag.DUEL_WIN
        : UserChallengeUpdateFlag.DUEL_LOSS
    )
    await UsageEventService.addEvent(client, {
      type: UsageEventType.DUEL_END_EVENT,
      target: userChallenge.duel.id,
      result: winChallenge ? 'win' : 'loss',
      startDate: userChallenge.startDate,
    })
    await UsageEventService.addEvent(client, {
      type: UsageEventType.CHALLENGE_END_EVENT,
      target: userChallenge.id,
      startDate: userChallenge.startDate,
    })
    dispatch(updateUserChallengeList(updatedChallenge))
    dispatch(unlockNextUserChallenge(updatedChallenge))
    dispatch(toggleChallengeDuelNotification(false))

    if (
      userChallenge.id == userChallengeList[userChallengeList.length - 1].id
    ) {
      setIsLastDuel(true)
    } else {
      navigate('/challenges')
    }
  }, [
    client,
    userChallenge,
    winChallenge,
    dispatch,
    userChallengeList,
    navigate,
  ])

  const setLastResult = useCallback(async () => {
    navigate('/challenges')
  }, [navigate])

  useEffect(() => {
    function handleResize() {
      const maxWidth = 940
      const maxHeight = 300
      const _width = chartContainer.current
        ? chartContainer.current.offsetWidth > maxWidth
          ? maxWidth
          : chartContainer.current.offsetWidth
        : 400
      setWidth(_width)
      const _height = chartContainer.current
        ? chartContainer.current.offsetHeight > maxHeight
          ? maxHeight
          : chartContainer.current.offsetHeight
        : 300
      setHeight(_height)
    }
    handleResize()
    window.addEventListener('resize', handleResize)
    return () => window.removeEventListener('resize', handleResize)
  }, [])

  useEffect(() => {
    let subscribed = true
    async function setChallengeResult() {
      const { isDone, isWin } = await challengeService.isChallengeDone(
        userChallenge,
        currentDataload
      )
      if (subscribed) {
        setResultModal(isDone)
        setWinChallenge(isWin)
      }
    }
    setChallengeResult()
    return () => {
      subscribed = false
    }
  }, [challengeService, client, currentDataload, userChallenge])

  useEffect(() => {
    let subscribed = true
    async function populateData() {
      if (isFinished) {
        const dataloads: Dataload[] =
          await challengeService.getUserChallengeDataload(userChallenge)
        if (subscribed) {
          setFinishedDataLoad(dataloads)
        }
      }
    }
    populateData()
    return () => {
      subscribed = false
    }
  }, [challengeService, dispatch, isFinished, userChallenge])

  return (
    <>
      <div className="duel-ongoing-container">
        <div className="duel-title text-16-normal">{duel.title}</div>
        {duel.state === UserDuelState.DONE ? (
          <div className="duel-goal text-26-bold">
            {userChallenge.success === UserChallengeSuccess.WIN
              ? t('duel.goal_done')
              : t('duel.goal_failed')}
          </div>
        ) : (
          <div className="duel-goal text-18-normal">
            {t('duel.goal1', {
              durationInDays,
              // eslint-disable-next-line camelcase
              smart_count: durationInDays,
            })}
            <span> </span>
            {t('duel.goal2', {
              title,
              // eslint-disable-next-line camelcase
              smart_count: title,
            })}
          </div>
        )}
        <div className="duel-consumption text-28-normal">
          <span className="consumption">{userConsumption}</span>
          {` / ${average} €`}
        </div>
        <div className="duel-chart fs-root" ref={chartContainer}>
          <DuelChart
            userChallenge={userChallenge}
            finishedDataLoad={finishedDataLoad}
            width={width}
            height={height}
          />
        </div>
        <div className="duel-chart-caption text-15-normal">
          <div className="duel-caption">
            <StyledIcon
              className="caption-icon"
              icon={CaptionAverageIcon}
              size={25}
            />
            <div className="caption-label">{t('duel.caption_average')}</div>
          </div>
          <div className="duel-caption">
            <StyledIcon
              className="caption-icon"
              icon={CaptionConsumptionIcon}
              size={25}
            />
            <div className="caption-label">{t('duel.caption_consumption')}</div>
          </div>
          <div className="duel-caption">
            <StyledIcon
              className="caption-icon"
              icon={CaptionIncomingIcon}
              size={25}
            />
            <div className="caption-label">{t('duel.caption_incoming')}</div>
          </div>
        </div>
      </div>
      <DuelResultModal
        open={resultModal}
        userChallenge={userChallenge}
        win={winChallenge}
        handleCloseClick={setResult}
      />
      <LastDuelModal open={isLastDuel} handleCloseClick={setLastResult} />
    </>
  )
}

export default DuelOngoing
