import Content from 'components/Content/Content'
import EcogestureSelectionDetail from 'components/EcogestureSelection/EcogestureSelectionDetail'
import EcogestureSelectionEnd from 'components/EcogestureSelection/EcogestureSelectionEnd'
import EcogestureSelectionModal from 'components/EcogestureSelection/EcogestureSelectionModal'
import EcogestureSelectionRestart from 'components/EcogestureSelection/EcogestureSelectionRestart'
import CozyBar from 'components/Header/CozyBar'
import Header from 'components/Header/Header'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { Ecogesture } from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import { useSelector } from 'react-redux'
import { useNavigate } from 'react-router-dom'
import EcogestureService from 'services/ecogesture.service'
import { AppStore } from 'store'
import './ecogestureSelection.scss'

const EcogestureSelection: React.FC = () => {
  const { t } = useI18n()
  const client = useClient()
  const navigate = useNavigate()
  const [isLoading, setIsLoading] = useState(true)
  const [headerHeight, setHeaderHeight] = useState<number>(0)
  const [indexEcogesture, setIndexEcogesture] = useState<number>(0)
  const [ecogestureList, setEcogestureList] = useState<Ecogesture[]>([])
  const [totalViewed, setTotalViewed] = useState<number>(0)
  const [totalAvailable, setTotalAvailable] = useState<number>(0)
  const [openEcogestureSelectionModal, setOpenEcogestureSelectionModal] =
    useState(false)

  const defineHeaderHeight = useCallback((height: number) => {
    setHeaderHeight(height)
  }, [])

  const ecogestureService = useMemo(
    () => new EcogestureService(client),
    [client]
  )
  const profileEcogesture = useSelector(
    (state: AppStore) => state.ecolyo.profileEcogesture
  )

  const getTitle = useCallback((): string => {
    if (
      indexEcogesture <= ecogestureList.length - 1 &&
      ecogestureList[indexEcogesture]
    ) {
      return ecogestureList[indexEcogesture].shortName
    }
    return t('ecogesture_selection.header')
  }, [ecogestureList, indexEcogesture, t])

  const validateChoice = useCallback(
    async (objective: boolean, doing: boolean) => {
      const updatedEcogesture: Ecogesture =
        await ecogestureService.updateEcogesture({
          ...ecogestureList[indexEcogesture],
          objective: objective,
          doing: doing,
          viewedInSelection: true,
        })
      const updatedList: Ecogesture[] = ecogestureList
      updatedList[indexEcogesture] = updatedEcogesture
      setEcogestureList(updatedList)
      setIndexEcogesture(prev => prev + 1)
    },
    [ecogestureList, ecogestureService, indexEcogesture]
  )

  const restartSelection = useCallback(async () => {
    setIsLoading(true)
    const availableList: Ecogesture[] =
      await ecogestureService.getEcogestureListByProfile(profileEcogesture)
    const filteredList: Ecogesture[] = availableList.filter(
      (ecogesture: Ecogesture) => ecogesture.viewedInSelection === false
    )
    const slicedFilteredList = filteredList.slice(0, 10)
    setTotalViewed(availableList.length - filteredList.length)
    setEcogestureList(slicedFilteredList)
    setIndexEcogesture(0)
    setIsLoading(false)
  }, [ecogestureService, profileEcogesture])

  useEffect(() => {
    let subscribed = true
    async function getFilteredList() {
      const availableList: Ecogesture[] =
        await ecogestureService.getEcogestureListByProfile(profileEcogesture)
      const filteredList: Ecogesture[] = availableList.filter(
        (ecogesture: Ecogesture) => ecogesture.viewedInSelection === false
      )
      const slicedFilteredList = filteredList.slice(0, 10)
      if (subscribed) {
        if (
          availableList.length === filteredList.length &&
          slicedFilteredList.length > 0
        ) {
          setOpenEcogestureSelectionModal(true)
        }
        setTotalAvailable(availableList.length)
        setTotalViewed(availableList.length - filteredList.length)
        setEcogestureList(slicedFilteredList)
        setIndexEcogesture(0)
        setIsLoading(false)
      }
    }
    getFilteredList()
    return () => {
      subscribed = false
    }
  }, [ecogestureService, profileEcogesture])

  if (isLoading) {
    return (
      <Content height={headerHeight}>
        <div className="eg-selection-loader">
          <Loader />
        </div>
      </Content>
    )
  }

  return (
    <>
      <CozyBar
        titleKey={'common.title_ecogestures_choice'}
        displayBackArrow={true}
        backFunction={() => navigate('/ecogestures')}
      />
      <Header
        setHeaderHeight={defineHeaderHeight}
        desktopTitleKey={'common.title_ecogestures_choice'}
        displayBackArrow={true}
      >
        <div className="eg-selection-header">
          {indexEcogesture <= ecogestureList.length - 1
            ? `${totalViewed + indexEcogesture + 1}/${totalAvailable}`
            : ''}
        </div>
      </Header>
      <Content height={headerHeight}>
        {indexEcogesture <= ecogestureList.length - 1 ? (
          <EcogestureSelectionDetail
            ecogesture={ecogestureList[indexEcogesture]}
            validate={validateChoice}
            title={getTitle()}
          />
        ) : totalAvailable > totalViewed + ecogestureList.length ? (
          <EcogestureSelectionRestart
            listLength={ecogestureList.length}
            restart={restartSelection}
          />
        ) : (
          <EcogestureSelectionEnd />
        )}
      </Content>
      {openEcogestureSelectionModal && (
        <EcogestureSelectionModal
          open={openEcogestureSelectionModal}
          handleCloseClick={() => setOpenEcogestureSelectionModal(false)}
        />
      )}
    </>
  )
}

export default EcogestureSelection
