import * as Sentry from '@sentry/react'
import { Client, useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidSlugType } from 'enum/fluidSlug.enum'
import { UsageEventType } from 'enum/usageEvent.enum'
import {
  AccountAuthData,
  AccountSgeData,
  FluidConnection,
  FluidStatus,
  UsageEvent,
} from 'models'
import { Dispatch, useCallback, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import AccountService from 'services/account.service'
import ConnectionService from 'services/connection.service'
import UsageEventService from 'services/usageEvent.service'
import { AppActionsTypes, AppStore } from 'store'
import { setLoading } from 'store/chart/chart.actions'
import { updatedFluidConnection } from 'store/global/global.actions'
import logApp from 'utils/logger'

const useKonnectorAuth = (
  fluidStatus: FluidStatus,
  login?: string,
  password?: string
): [() => Promise<null | undefined>, () => Promise<void>, string] => {
  const client: Client = useClient()
  const { t } = useI18n()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const konnectorSlug: FluidSlugType =
    fluidStatus.connection.konnectorConfig.slug
  const { sgeConnect } = useSelector((state: AppStore) => state.ecolyo.global)

  const [connectError, setError] = useState<string>('')

  const onSuccess = useCallback(async () => {
    const updatedConnection: FluidConnection = {
      ...fluidStatus.connection,
      shouldLaunchKonnector: true,
    }
    dispatch(updatedFluidConnection(fluidStatus.fluidType, updatedConnection))
  }, [dispatch, fluidStatus.fluidType, fluidStatus.connection])

  const sendUsageEventError = async (slug: string): Promise<UsageEvent> => {
    return UsageEventService.addEvent(client, {
      type: UsageEventType.KONNECTOR_CONNECT_EVENT,
      target: slug,
      result: 'error',
    })
  }

  const connect = async () => {
    const connectionService = new ConnectionService(client)
    try {
      // If first connexion, send the usage event
      await UsageEventService.addEvent(client, {
        type: UsageEventType.KONNECTOR_ATTEMPT_EVENT,
        target: konnectorSlug,
        result: 'error',
      })
      const { account: _account, trigger: _trigger } =
        konnectorSlug === FluidSlugType.WATER
          ? await connectionService.connectNewUser(konnectorSlug, {
              login: login ? login : '',
              password: password,
            })
          : await connectionService.connectNewUser(
              konnectorSlug,
              undefined,
              sgeConnect
            )
      if (!_trigger || !_account) {
        setError(t('konnector_form.error_account_creation'))
        sendUsageEventError(konnectorSlug)
        return null
      }
      const updatedConnection: FluidConnection = {
        ...fluidStatus.connection,
        account: _account,
        trigger: _trigger,
      }
      setLoading(false)
      dispatch(updatedFluidConnection(fluidStatus.fluidType, updatedConnection))
      onSuccess()
    } catch (error) {
      setLoading(false)
      sendUsageEventError(konnectorSlug)
      logApp.error(error)
      Sentry.captureException(JSON.stringify({ error }))
    }
  }

  const update = async () => {
    if (fluidStatus.connection.account) {
      const _account = fluidStatus.connection.account
      let auth: AccountAuthData | AccountSgeData
      if (konnectorSlug === FluidSlugType.WATER) {
        auth = {
          login: login,
          password: password,
        } as AccountAuthData
      } else {
        auth = {
          pointId: sgeConnect.pdl ? sgeConnect.pdl.toString() : '',
          firstname: sgeConnect.firstName,
          lastname: sgeConnect.lastName,
          address: sgeConnect.address,
          postalCode: sgeConnect.zipCode ? sgeConnect.zipCode.toString() : '',
          city: sgeConnect.city,
        } as AccountSgeData
      }
      _account.auth = auth
      const accountService = new AccountService(client)
      const updatedAccount = await accountService.updateAccount(_account)
      const updatedConnection: FluidConnection = {
        ...fluidStatus.connection,
        account: updatedAccount,
      }
      dispatch(updatedFluidConnection(fluidStatus.fluidType, updatedConnection))
      onSuccess()
    }
  }
  return [connect, update, connectError]
}

export default useKonnectorAuth
