import {
  Accordion,
  AccordionDetails,
  AccordionSummary,
  Button,
} from '@material-ui/core'
import chevronDown from 'assets/icons/ico/chevron-down.svg'
import exportIcon from 'assets/icons/ico/export.svg'
import classNames from 'classnames'
import ExportDoneModal from 'components/Export/exportDoneModal'
import ExportLoadingModal from 'components/Export/exportLoadingModal'
import ExportStartModal from 'components/Export/exportStartModal'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { remove } from 'lodash'
import React, { useEffect, useMemo, useState } from 'react'
import ConsumptionDataManager from 'services/consumption.service'
import './exportData.scss'

const ExportData: React.FC = () => {
  const { t } = useI18n()
  const client = useClient()
  const consumptionService = useMemo(
    () => new ConsumptionDataManager(client),
    [client]
  )

  const [isExportStartModal, setIsExportStartModal] = useState<boolean>(false)
  const [isExportLoadingModal, setIsExportLoadingModal] =
    useState<boolean>(false)
  const [isExportDoneModal, setIsExportDoneModal] = useState<boolean>(false)
  const [hasError, setHasError] = useState<boolean>(false)
  const [exportableFluids, setExportableFluids] = useState<FluidType[]>([])
  const [answer, setAnswer] = useState<FluidType[]>([])

  const [active, setActive] = useState<boolean>(false)

  const toggleAccordion = () => {
    setActive(prev => !prev)
  }

  const handleChange = (value: FluidType) => {
    const tempAnswer = [...answer]
    if (tempAnswer.includes(value)) {
      remove(tempAnswer, answer => answer === value)
    } else {
      tempAnswer.push(value)
    }
    setAnswer(tempAnswer)
  }

  useEffect(() => {
    let subscribed = true
    const getExportableFluids = async () => {
      const exportableFluidsData: FluidType[] =
        await consumptionService.getExportableFluids(
          [FluidType.ELECTRICITY, FluidType.WATER, FluidType.GAS],
          TimeStep.MONTH
        )
      setExportableFluids(exportableFluidsData)
      setAnswer(exportableFluidsData)
      subscribed = false
    }

    if (subscribed) {
      getExportableFluids()
    }
    return () => {
      subscribed = false
    }
  }, [consumptionService])

  const fluidCheckbox = () =>
    exportableFluids.map((fluidType, key) => (
      <label
        key={key}
        className={classNames('checkbox', {
          ['answer-checked']: answer.includes(fluidType),
        })}
      >
        <input
          type={'checkbox'}
          value={fluidType}
          name={t('FLUID.' + FluidType[fluidType] + '.LABEL')}
          onChange={() => handleChange(fluidType)}
          checked={answer.includes(fluidType)}
        />
        {t('FLUID.' + FluidType[fluidType] + '.LABEL')}
      </label>
    ))

  const handleDone = (e?: any) => {
    if (e) {
      setHasError(true)
    }
    setIsExportDoneModal(true)
    setIsExportLoadingModal(false)
  }

  return (
    <>
      <div className="export-option-root">
        <div className="export-option-content">
          <Accordion
            expanded={active}
            onChange={toggleAccordion}
            classes={{
              root: 'expansion-panel-root',
            }}
          >
            <AccordionSummary
              aria-label={t('profile_type.accessibility.button_toggle_export')}
              expandIcon={
                <Icon icon={chevronDown} size={16} className="accordion-icon" />
              }
              classes={{
                root: 'expansion-panel-summary',
                content: 'expansion-panel-content',
              }}
            >
              <Icon className="export-icon" icon={exportIcon} size={42} />
              <div className="text-16-normal accordion-title">
                {t('export.title_export')}
              </div>
            </AccordionSummary>
            <AccordionDetails
              classes={{
                root: 'expansion-panel-details',
              }}
            >
              <div className="text-15-normal content intro">
                {t('export.text1')}
              </div>
              <div className="text-16-bold content">
                {t('export.fluid_select')}
              </div>

              {exportableFluids.length === 0 ? (
                <div className="text-15-normal content intro">
                  {t('export.no_data')}
                </div>
              ) : (
                <>
                  {fluidCheckbox()}
                  <Button
                    aria-label={t('unsubscribe.button_accessibility')}
                    onClick={() => setIsExportStartModal(true)}
                    classes={{
                      root: 'btn-secondary-negative',
                      label: 'text-16-normal',
                    }}
                    type="submit"
                    disabled={answer.length === 0}
                  >
                    {t('export.button_download')}
                  </Button>
                </>
              )}
            </AccordionDetails>
          </Accordion>
        </div>
      </div>
      <ExportStartModal
        open={isExportStartModal}
        handleCloseClick={() => setIsExportStartModal(false)}
        handleDownloadClick={() => {
          setIsExportStartModal(false)
          setIsExportLoadingModal(true)
        }}
      />
      {isExportLoadingModal && (
        <ExportLoadingModal
          open={isExportLoadingModal}
          handleCloseClick={() => {
            setIsExportLoadingModal(false)
          }}
          handleDone={(e?: any): void => handleDone(e)}
          selectedFluids={answer}
        />
      )}
      <ExportDoneModal
        open={isExportDoneModal}
        error={hasError}
        handleCloseClick={() => setIsExportDoneModal(false)}
      />
    </>
  )
}

export default ExportData
