/* eslint-disable camelcase */
import * as Sentry from '@sentry/react'
import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import {
  createAccount,
  deleteAccount,
  fetchAccount,
  updateAccount,
} from 'cozy-harvest-lib/dist/connections/accounts'
import { build } from 'cozy-harvest-lib/dist/helpers/accounts'
import logger from 'cozy-logger'
import { ACCOUNTS_DOCTYPE } from 'doctypes'
import { DateTime } from 'luxon'
import {
  Account,
  AccountAttributes,
  AccountAuthData,
  AccountSgeData,
  Konnector,
  Trigger,
} from 'models'
import logApp from 'utils/logger'
import TriggerService from './triggers.service'

const logStack = logger.namespace('accountService')

export default class AccountService {
  private _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  private buildAccountAttributes(
    konnector: Konnector,
    authData: AccountAuthData | AccountSgeData
  ): AccountAttributes {
    return build(konnector, authData)
  }

  public async createAccount(
    konnector: Konnector,
    accountAuthData: AccountAuthData | AccountSgeData
  ): Promise<Account> {
    const accountAttributes = this.buildAccountAttributes(
      konnector,
      accountAuthData
    )
    return createAccount(this._client, konnector, accountAttributes)
  }

  public async getAccount(id: string): Promise<Account> {
    try {
      const account: Account = await fetchAccount(this._client, id)
      return account
    } catch (error) {
      const errorMessage = `Get account failed :${JSON.stringify(error)}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw new Error('Get account failed')
    }
  }

  public async getAccountByType(type: string): Promise<Account | null> {
    try {
      const query: QueryDefinition = Q(ACCOUNTS_DOCTYPE).where({
        account_type: type,
      })
      // .indexFields(['account_type'])
      const { data: accounts }: QueryResult<Account[]> =
        await this._client.query(query)
      if (accounts.length > 1) {
        // If several account are found we will used trigger date to select the older
        const triggerService = new TriggerService(this._client)
        const triggers: (Trigger | null)[] = await Promise.all(
          accounts.map((_account: Account) =>
            triggerService.getTriggerForAccount(_account)
          )
        )
        // Keep the accountid which have the older trigger
        let olderDate: DateTime = DateTime.now()
        let olderAccountId: string | null = null
        for (const _trigger of triggers) {
          if (
            _trigger?.cozyMetadata?.createdAt &&
            DateTime.fromISO(_trigger.cozyMetadata.createdAt, {
              zone: 'utc',
            }) < olderDate
          ) {
            olderDate = DateTime.fromISO(_trigger.cozyMetadata.createdAt, {
              zone: 'utc',
            })
            olderAccountId = _trigger.message.account
          }
        }
        // Retrieve the accountId and return it
        const filteredAccounts: Account[] = accounts.filter(
          _account => _account._id === olderAccountId
        )
        return filteredAccounts[0] ? filteredAccounts[0] : null
      } else {
        return accounts[0] ? accounts[0] : null
      }
    } catch (error) {
      const errorMessage = `Error GetAccountByType: ${JSON.stringify(error)}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      return null
    }
  }

  public async getAccountsByType(type: string): Promise<Account[]> {
    try {
      const query: QueryDefinition = Q(ACCOUNTS_DOCTYPE).where({
        account_type: type,
      })
      // .indexFields(['account_type'])
      const { data: accounts }: QueryResult<Account[]> =
        await this._client.query(query)
      return accounts
    } catch (error) {
      const errorMessage = `Error: GetAccountsByType: ${JSON.stringify(error)}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      return []
    }
  }

  public async updateAccount(account: Account): Promise<Account> {
    try {
      const updatedAccount: Account = await updateAccount(this._client, account)
      return updatedAccount
    } catch (error) {
      const errorMessage = `Update account failed: ${JSON.stringify(error)}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw new Error('Update account failed')
    }
  }

  public async deleteAccount(account: Account): Promise<boolean> {
    try {
      await deleteAccount(this._client, account)
      return true
    } catch (error) {
      const errorMessage = `Delete account failed`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw new Error(errorMessage)
    }
  }

  public async createIndexAccount(): Promise<[]> {
    const query: QueryDefinition = Q(ACCOUNTS_DOCTYPE)
      .where({ account_type: 'index' })
      // .indexFields(['account_type'])
      .limitBy(1)
    const { data: result }: QueryResult<[]> = await this._client.query(query)
    return result
  }
}
