import * as Sentry from '@sentry/react'
import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import logger from 'cozy-logger'
import { ECOGESTURE_DOCTYPE } from 'doctypes'
import { Season, Usage } from 'enum/ecogesture.enum'
import { FluidType } from 'enum/fluid.enum'
import { IndividualOrCollective, WarmingType } from 'enum/profileType.enum'
import { orderBy } from 'lodash'
import { Ecogesture } from 'models'
import { ProfileEcogesture } from 'models/profileEcogesture.model'
import logApp from 'utils/logger'

const logStack = logger.namespace('ecogestureService')

export default class EcogestureService {
  private readonly _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  public async getAllEcogestures(
    seasonFilter?: Season,
    orderByID?: boolean
  ): Promise<Ecogesture[]> {
    let query: QueryDefinition = Q(ECOGESTURE_DOCTYPE)
    if (seasonFilter && seasonFilter !== Season.NONE) {
      query = query
        .where({ season: { $ne: seasonFilter } })
        .sortBy([{ season: 'desc' }])
    } else if (orderByID) {
      query = query.where({}).sortBy([{ _id: 'asc' }])
    } else {
      query = query.where({}).sortBy([{ season: 'desc' }])
    }

    const { data: ecogestures }: QueryResult<Ecogesture[]> =
      await this._client.query(query)

    if (seasonFilter && seasonFilter !== Season.NONE) {
      const { data: ecogesturesWithSeason }: QueryResult<Ecogesture[]> =
        await this._client.query(
          Q(ECOGESTURE_DOCTYPE)
            .where({ season: { $eq: seasonFilter } })
            .sortBy([{ season: 'asc' }])
        )
      return [...ecogesturesWithSeason, ...ecogestures]
    }
    return ecogestures
  }
  /**
   * @param {string} ids - ecogestures ids
   * @returns {Promise<Ecogesture[]>}
   */
  public async getEcogesturesByIds(ids: string[]): Promise<Ecogesture[]> {
    const query: QueryDefinition = Q(ECOGESTURE_DOCTYPE).getByIds(ids)
    const { data: ecogestures }: QueryResult<Ecogesture[]> =
      await this._client.query(query)
    return ecogestures
  }

  public async deleteAllEcogestures(): Promise<boolean> {
    const ecogestures = await this.getAllEcogestures()
    try {
      for (const ecogesture of ecogestures) {
        await this._client.destroy(ecogesture)
      }
      return true
    } catch (error) {
      const errorMessage = `Error deleteAllEcogestures: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw error
    }
  }

  public async reinitAllEcogestures(): Promise<boolean> {
    const ecogestures = await this.getAllEcogestures()
    try {
      for (const ecogesture of ecogestures) {
        await this.updateEcogesture({
          ...ecogesture,
          objective: false,
          doing: false,
          viewedInSelection: false,
        })
      }
      return true
    } catch (error) {
      const errorMessage = `Error reinitAllEcogestures: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw error
    }
  }

  /**
   * Removes ecogestures from the list that doesn't fit with user's usages
   * @param {Ecogesture[]} ecogestureList
   * @param {ProfileEcogesture} profileEcogesture
   * @returns {Ecogesture[]}
   */
  public filterByUsage(
    ecogestureList: Ecogesture[],
    profileEcogesture: ProfileEcogesture
  ): Ecogesture[] {
    const filteredByUsage: Ecogesture[] = ecogestureList.filter(ecogesture => {
      switch (ecogesture.usage) {
        case Usage.HEATING:
          if (
            ecogesture.fluidTypes.includes(FluidType.ELECTRICITY) &&
            profileEcogesture.heating === IndividualOrCollective.INDIVIDUAL &&
            profileEcogesture.warmingFluid === WarmingType.ELECTRICITY
          ) {
            return true
          } else if (
            ecogesture.fluidTypes.includes(FluidType.GAS) &&
            profileEcogesture.heating === IndividualOrCollective.INDIVIDUAL &&
            profileEcogesture.warmingFluid === WarmingType.GAS
          ) {
            return true
          } else {
            return false
          }
        case Usage.ECS:
          if (
            profileEcogesture.hotWater === IndividualOrCollective.INDIVIDUAL
          ) {
            return true
          } else {
            return false
          }
        default:
          return true
      }
    })
    return filteredByUsage
  }

  /**
   * Removes ecogesture from the list that depends on equipment the user hasn't
   * @param {Ecogesture[]} ecogestureList
   * @param {ProfileEcogesture} profileEcogesture
   * @returns {Ecogesture[]}
   */
  public filterByEquipment(
    ecogestureList: Ecogesture[],
    profileEcogesture: ProfileEcogesture
  ): Ecogesture[] {
    for (const ecogesture of ecogestureList) {
      if (ecogesture.equipment === true) {
        for (const equipmentType of ecogesture.equipmentType) {
          if (!profileEcogesture.equipments.includes(equipmentType)) {
            const index = ecogestureList.indexOf(ecogesture)
            ecogestureList.splice(index, 1)
          }
        }
      }
    }
    return ecogestureList
  }

  /**
   * Return a filtered list according to ecogesture profile, the list is sorted by low difficulty and high efficiency
   * @param {ProfileEcogesture} profileEcogesture
   * @returns {Ecogesture[]}
   */
  public async getEcogestureListByProfile(
    profileEcogesture: ProfileEcogesture
  ): Promise<Ecogesture[]> {
    const ecogestureList: Ecogesture[] = await this.getAllEcogestures()
    const filteredByUsage: Ecogesture[] = this.filterByUsage(
      ecogestureList,
      profileEcogesture
    )
    const filteredByEquipment: Ecogesture[] = this.filterByEquipment(
      filteredByUsage,
      profileEcogesture
    )
    const filteredFlaggedEcogesture: Ecogesture[] = filteredByEquipment.filter(
      (ecogesture: Ecogesture) =>
        (ecogesture.objective === false &&
          ecogesture.doing === false &&
          ecogesture.viewedInSelection === false) ||
        ecogesture.viewedInSelection === true
    )
    const sortedByDifficultyAndEfficiency: Ecogesture[] = orderBy(
      filteredFlaggedEcogesture,
      [
        ecogesture => {
          return ecogesture.difficulty
        },
        ecogesture => {
          return ecogesture.efficiency
        },
      ],
      ['asc', 'desc']
    )
    return sortedByDifficultyAndEfficiency
  }

  /**
   * Update one ecogesture
   * @param {Ecogesture} ecogesture - Ecogesture to save
   * @returns {Ecogesture} Udpated Ecogesture
   */
  public async updateEcogesture(ecogesture: Ecogesture): Promise<Ecogesture> {
    const { data: updatedEcogesture }: QueryResult<Ecogesture> =
      await this._client.save(ecogesture)
    return updatedEcogesture
  }
}
