import * as Sentry from '@sentry/react'
import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import {
  REMOTE_ORG_ECOLYO_AGENT,
  REMOTE_ORG_ECOLYO_AGENT_REC,
} from 'doctypes/remote/org.ecolyo.agent'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import get from 'lodash/get'
import { DateTime } from 'luxon'
import mjml2html from 'mjml'
import { PerformanceIndicator } from 'models'
import { MonthlyReport } from 'models/monthlyReport.model'
import ConsumptionService from 'services/consumption.service'
import EnvironmentService from 'services/environment.service'
import MailService from 'services/mail.service'
import ProfileService from 'services/profile.service'
import { getMonthNameWithPrep } from 'utils/utils'
import { runService } from './service'
const monthlyReportTemplate = require('notifications/monthlyReport.hbs')

const logStack = logger.namespace('report')

interface MonthlyReportNotificationProps {
  client: Client
}

/**
 * Get consumption value for all fluid if exist
 * @param client
 * @param fluidType - FluidType
 */
const getConsumptionValue = async (
  client: Client,
  fluidType: FluidType[]
): Promise<PerformanceIndicator[]> => {
  const consumptionService = new ConsumptionService(client)
  const analysisDate = DateTime.local().setZone('utc', { keepLocalTime: true })
  const periods = {
    timePeriod: {
      startDate: analysisDate.minus({ month: 1 }).startOf('month'),
      endDate: analysisDate.minus({ month: 1 }).endOf('month'),
    },
    comparisonTimePeriod: {
      startDate: analysisDate.minus({ month: 2 }).startOf('month'),
      endDate: analysisDate.minus({ month: 2 }).endOf('month'),
    },
  }
  return consumptionService.getPerformanceIndicators(
    periods.timePeriod,
    TimeStep.MONTH,
    fluidType,
    periods.comparisonTimePeriod
  )
}

/**
 * Build consumption text to attach in email. According to client consumption, display electricity and/or gaz and/or water
 * @param client
 * @returns string
 */
const buildConsumptionText = async (client: Client) => {
  logStack('info', 'Building consumption text...')
  const consumption = await getConsumptionValue(client, [
    FluidType.ELECTRICITY,
    FluidType.GAS,
    FluidType.WATER,
  ])
  let text = ''
  if (consumption[FluidType.ELECTRICITY]) {
    const value = consumption[FluidType.ELECTRICITY].percentageVariation
      ? consumption[FluidType.ELECTRICITY].percentageVariation
      : 0
    if (value) {
      if (value > 0) {
        text +=
          '<span class="elec-text"><br>+&nbsp;' +
          Math.ceil(value * 100) +
          "&nbsp;%&nbsp;d'électricité</span>"
      } else {
        text +=
          '<span class="elec-text"><br>-&nbsp;' +
          Math.ceil(Math.abs(value * 100)) +
          "&nbsp;%&nbsp;d'électricité</span>"
      }
    }
  }
  if (consumption[FluidType.GAS]) {
    const value =
      consumption[FluidType.GAS]?.percentageVariation !== null
        ? consumption[FluidType.GAS].percentageVariation
        : 0
    if (value) {
      if (value > 0) {
        text +=
          '<span class="gas-text"><br>+&nbsp;' +
          Math.ceil(value * 100) +
          '&nbsp;%&nbsp;de&nbsp;gaz</span>'
      } else {
        text +=
          '<span class="gas-text"><br>-&nbsp;' +
          Math.ceil(Math.abs(value * 100)) +
          '&nbsp;%&nbsp;de&nbsp;gaz</span>'
      }
    }
  }
  if (consumption[FluidType.WATER]) {
    const value = consumption[FluidType.WATER].percentageVariation
      ? consumption[FluidType.WATER].percentageVariation
      : 0
    if (value) {
      if (value > 0) {
        text +=
          '<span class="water-text"><br>+&nbsp;' +
          Math.ceil(value * 100) +
          "&nbsp;%&nbsp;d'eau</span>"
      } else {
        text +=
          '<span class="water-text"><br>-&nbsp;' +
          Math.ceil(Math.abs(value * 100)) +
          "&nbsp;%&nbsp;d'eau</span>"
      }
    }
  }
  return text
}

/**
 * getMonthlyReport
 */
const getMonthlyReport = async (
  year: string,
  month: string,
  client: Client
): Promise<MonthlyReport> => {
  try {
    const environmentService = new EnvironmentService()
    logStack(
      'info',
      environmentService.isProduction()
        ? 'Fetching data from BO prod'
        : 'Fetching data from BO rec'
    )

    const result = await client
      .getStackClient()
      .fetchJSON(
        'GET',
        environmentService.isProduction()
          ? `${REMOTE_ORG_ECOLYO_AGENT}?year=${year}&month=${month}`
          : `${REMOTE_ORG_ECOLYO_AGENT_REC}?year=${year}&month=${month}`
      )
    return result
  } catch (error) {
    logStack('error', JSON.stringify(error))
    Sentry.captureException(JSON.stringify({ error }))
    return {
      year: parseInt(year),
      month: parseInt(month),
      subject: '',
      info: '',
      image: '',
      newsTitle: 'Les nouveautés du service',
      newsContent: '',
      question: '',
      link: '',
    }
  }
}

const monthlyReportNotification = async ({
  client,
}: MonthlyReportNotificationProps) => {
  logStack('info', 'Fetching user profile...')
  const upm = new ProfileService(client)
  let userProfil = await upm.getProfile()
  if (!userProfil?.sendAnalysisNotification) {
    logStack(
      'info',
      'End of process - Report Notification disabled in user profile'
    )
    return
  }

  // Init mail token for user in case he don't have one
  if (!userProfil.mailToken || userProfil.mailToken === '') {
    const token: string = require('crypto').randomBytes(48).toString('hex')

    try {
      await upm.updateProfile({
        mailToken: token,
      })
    } catch (error) {
      logStack('error', 'Update mailToken user profile error : ' + error)
      Sentry.captureException(JSON.stringify({ error }))
      throw error
    }
  }

  let username = ''
  let url = ''

  logStack('info', 'Fetching data for mail...')
  // Retrieve public name from the stack
  const settings = await client
    .getStackClient()
    .fetchJSON('GET', '/settings/instance')
  const publicName = get(settings, 'data.attributes.public_name')
  if (publicName) {
    username = publicName
  }

  // Retrieve link to ecolyo app from the stack
  const apps = await client.getStackClient().fetchJSON('GET', '/apps/ecolyo')
  const appLink = get(apps, 'data.links.related')
  if (appLink) {
    url = appLink
  }

  logStack('info', 'Creation of mail...')
  const mailService = new MailService()

  const today = DateTime.local().setZone('utc', {
    keepLocalTime: true,
  })
  const month = today.toFormat('MM')
  const year = today.toFormat('yyyy')
  const monthlyReport: MonthlyReport = await getMonthlyReport(
    year,
    month,
    client
  )

  let unsubscribeUrl
  userProfil = await upm.getProfile()
  let token = undefined
  if (userProfil?.mailToken) {
    token = userProfil.mailToken
  }

  if (!url.includes('analysis')) {
    unsubscribeUrl = url + '/#/unsubscribe'
    url = url + '/#/analysis'
    if (token) {
      url += '?token=' + token
    }
  } else {
    unsubscribeUrl = url.replace('analysis', 'unsubscribe')
  }

  const consumptionText = await buildConsumptionText(client)

  const isInfo: boolean = monthlyReport.info !== ''

  const isServiceNews: boolean =
    monthlyReport.newsTitle !== '' && monthlyReport.newsContent !== ''

  const isPoll: boolean =
    monthlyReport.question !== '' && monthlyReport.link !== ''

  const date = DateTime.local()
    .setZone('utc', { keepLocalTime: true })
    .minus({ month: 1 })

  const environmentService = new EnvironmentService()
  const baseUrl = environmentService.getPublicURL()

  const template = monthlyReportTemplate({
    title: 'Du nouveau dans votre espace Ecolyo !',
    baseUrl: baseUrl,
    username: username,
    clientUrl: url,
    unsubscribeUrl: unsubscribeUrl,
    consumptionTextExist: consumptionText.length > 0,
    consumptionText: consumptionText.replace(/{cozyUrl}/g, appLink + '#/'),
    infoText: monthlyReport.info.replace(/{cozyUrl}/g, appLink + '#/'),
    infoImage:
      monthlyReport.image !== ''
        ? baseUrl + monthlyReport.image
        : baseUrl + '/assets/astuce.png',
    isContent: isInfo || isServiceNews || isPoll,
    isInfo: isInfo,
    divider1: (isInfo && isServiceNews) || (isInfo && isPoll),
    isServiceNews: isServiceNews,
    divider2: isServiceNews && isPoll,
    isPoll: isPoll,
    newsTitle: monthlyReport.newsTitle.replace(/{cozyUrl}/g, appLink + '#/'),
    newsContent: monthlyReport.newsContent.replace(
      /{cozyUrl}/g,
      appLink + '#/'
    ),
    pollText: monthlyReport.question.replace(/{cozyUrl}/g, appLink + '#/'),
    pollUrl: monthlyReport.link,
    previousMonth: getMonthNameWithPrep(date.minus({ month: 1 })),
    consoImageUrl: baseUrl + '/assets/multifluidConsumption.png',
    feedbackImageUrl: baseUrl + '/assets/feedback.png',
  })

  const mailData = {
    mode: 'noreply',
    subject: monthlyReport.subject,
    parts: [
      {
        type: 'text/html',
        body: mjml2html(template).html,
      },
    ],
  }
  logStack('info', 'Sending mail...')
  mailService.SendMail(client, mailData)
}

runService(monthlyReportNotification)
